@@ -31,7 +31,6 @@
 #include "config.h"
 
 #include "glib.h"
-#include "galias.h"
 
 
 #define HASH_TABLE_MIN_SIZE 11
@@ -54,7 +53,6 @@ struct _GHashTable
   GHashNode      **nodes;
   GHashFunc        hash_func;
   GEqualFunc       key_equal_func;
-  volatile guint   ref_count;
   GDestroyNotify   key_destroy_func;
   GDestroyNotify   value_destroy_func;
 };
@@ -85,6 +83,13 @@ static guint g_hash_table_foreach_remove_or_steal (GHashTable     *hash_table,
                                                    gboolean        notify);
 
 
+#ifndef DISABLE_MEM_POOLS
+G_LOCK_DEFINE_STATIC (g_hash_global);
+
+static GMemChunk *node_mem_chunk = NULL;
+static GHashNode *node_free_list = NULL;
+#endif
+
 /**
  * g_hash_table_new:
  * @hash_func: a function to create a hash value from a key.
@@ -99,7 +104,7 @@ static guint g_hash_table_foreach_remove_or_steal (GHashTable     *hash_table,
  *   directly in a similar fashion to g_direct_equal(), but without the
  *   overhead of a function call.
  *
- * Creates a new #GHashTable with a reference count of 1.
+ * Creates a new #GHashTable.
  * 
  * Return value: a new #GHashTable.
  **/
@@ -122,9 +127,9 @@ g_hash_table_new (GHashFunc    hash_func,
  *   value used when removing the entry from the #GHashTable or %NULL if 
  *   you don't want to supply such a function.
  * 
- * Creates a new #GHashTable like g_hash_table_new() with a reference count
- * of 1 and allows to specify functions to free the memory allocated for the
- * key and value that get called when removing the entry from the #GHashTable.
+ * Creates a new #GHashTable like g_hash_table_new() and allows to specify
+ * functions to free the memory allocated for the key and value that get 
+ * called when removing the entry from the #GHashTable.
  * 
  * Return value: a new #GHashTable.
  **/
@@ -135,102 +140,47 @@ g_hash_table_new_full (GHashFunc       hash_func,
 		       GDestroyNotify  value_destroy_func)
 {
   GHashTable *hash_table;
+  guint i;
   
-  hash_table = g_slice_new (GHashTable);
+  hash_table = g_new (GHashTable, 1);
   hash_table->size               = HASH_TABLE_MIN_SIZE;
   hash_table->nnodes             = 0;
   hash_table->hash_func          = hash_func ? hash_func : g_direct_hash;
   hash_table->key_equal_func     = key_equal_func;
-  hash_table->ref_count          = 1;
   hash_table->key_destroy_func   = key_destroy_func;
   hash_table->value_destroy_func = value_destroy_func;
-  hash_table->nodes              = g_new0 (GHashNode*, hash_table->size);
+  hash_table->nodes              = g_new (GHashNode*, hash_table->size);
+  
+  for (i = 0; i < hash_table->size; i++)
+    hash_table->nodes[i] = NULL;
   
   return hash_table;
 }
 
-
-/**
- * g_hash_table_ref:
- * @hash_table: a valid #GHashTable.
- * 
- * Atomically increments the reference count of @hash_table by one.
- * This function is MT-safe and may be called from any thread.
- * 
- * Return value: the passed in #GHashTable.
- * 
- * Since: 2.10
- **/
-GHashTable*
-g_hash_table_ref (GHashTable *hash_table)
-{
-  g_return_val_if_fail (hash_table != NULL, NULL);
-  g_return_val_if_fail (hash_table->ref_count > 0, hash_table);
-
-  g_atomic_int_add (&hash_table->ref_count, 1);
-  return hash_table;
-}
-
-/**
- * g_hash_table_unref:
- * @hash_table: a valid #GHashTable.
- * 
- * Atomically decrements the reference count of @hash_table by one.
- * If the reference count drops to 0, all keys and values will be
- * destroyed, and all memory allocated by the hash table is released.
- * This function is MT-safe and may be called from any thread.
- * 
- * Since: 2.10
- **/
-void
-g_hash_table_unref (GHashTable *hash_table)
-{
-  g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
-
-  if (g_atomic_int_exchange_and_add (&hash_table->ref_count, -1) - 1 == 0)
-    {
-      gint i;
-
-      for (i = 0; i < hash_table->size; i++)
-        g_hash_nodes_destroy (hash_table->nodes[i], 
-                              hash_table->key_destroy_func,
-                              hash_table->value_destroy_func);
-      g_free (hash_table->nodes);
-      g_slice_free (GHashTable, hash_table);
-    }
-}
-
 /**
  * g_hash_table_destroy:
  * @hash_table: a #GHashTable.
  * 
- * Destroys all keys and values in the #GHashTable and decrements its
- * reference count by 1. If keys and/or values are dynamically allocated,
- * you should either free them first or create the #GHashTable with destroy
- * notifiers using g_hash_table_new_full(). In the latter case the destroy
- * functions you supplied will be called on all keys and values during the
- * destruction phase.
+ * Destroys the #GHashTable. If keys and/or values are dynamically 
+ * allocated, you should either free them first or create the #GHashTable
+ * using g_hash_table_new_full(). In the latter case the destroy functions 
+ * you supplied will be called on all keys and values before destroying 
+ * the #GHashTable.
  **/
 void
 g_hash_table_destroy (GHashTable *hash_table)
 {
-  gint i;
+  guint i;
   
   g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
   
   for (i = 0; i < hash_table->size; i++)
-    {
-      g_hash_nodes_destroy (hash_table->nodes[i], 
-                            hash_table->key_destroy_func,
-                            hash_table->value_destroy_func);
-      hash_table->nodes[i] = NULL;
-    }
-  hash_table->nnodes = 0;
-  hash_table->size = HASH_TABLE_MIN_SIZE;
-
-  g_hash_table_unref (hash_table);
+    g_hash_nodes_destroy (hash_table->nodes[i], 
+			  hash_table->key_destroy_func,
+			  hash_table->value_destroy_func);
+  
+  g_free (hash_table->nodes);
+  g_free (hash_table);
 }
 
 static inline GHashNode**
@@ -262,10 +212,7 @@ g_hash_table_lookup_node (GHashTable	*hash_table,
  * @hash_table: a #GHashTable.
  * @key: the key to look up.
  * 
- * Looks up a key in a #GHashTable. Note that this function cannot
- * distinguish between a key that is not present and one which is present
- * and has the value %NULL. If you need this distinction, use
- * g_hash_table_lookup_extended().
+ * Looks up a key in a #GHashTable.
  * 
  * Return value: the associated value, or %NULL if the key is not found.
  **/
@@ -342,7 +289,6 @@ g_hash_table_insert (GHashTable *hash_table,
   GHashNode **node;
   
   g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
   
   node = g_hash_table_lookup_node (hash_table, key);
   
@@ -391,7 +337,6 @@ g_hash_table_replace (GHashTable *hash_table,
   GHashNode **node;
   
   g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
   
   node = g_hash_table_lookup_node (hash_table, key);
   
@@ -543,7 +488,7 @@ g_hash_table_foreach_remove_or_steal (GHashTable *hash_table,
                                       gboolean    notify)
 {
   GHashNode *node, *prev;
-  gint i;
+  guint i;
   guint deleted = 0;
   
   for (i = 0; i < hash_table->size; i++)
@@ -596,7 +541,7 @@ g_hash_table_foreach_remove_or_steal (GHashTable *hash_table,
  * pair, and the given @user_data parameter.  The hash table may not
  * be modified while iterating over it (you can't add/remove
  * items). To remove all items matching a predicate, use
- * g_hash_table_foreach_remove().
+ * g_hash_table_remove().
  **/
 void
 g_hash_table_foreach (GHashTable *hash_table,
@@ -700,7 +645,28 @@ static GHashNode*
 g_hash_node_new (gpointer key,
 		 gpointer value)
 {
-  GHashNode *hash_node = g_slice_new (GHashNode);
+  GHashNode *hash_node;
+  
+#ifdef DISABLE_MEM_POOLS
+  hash_node = g_new (GHashNode, 1);
+#else
+  G_LOCK (g_hash_global);
+  if (node_free_list)
+    {
+      hash_node = node_free_list;
+      node_free_list = node_free_list->next;
+    }
+  else
+    {
+      if (!node_mem_chunk)
+	node_mem_chunk = g_mem_chunk_new ("hash node mem chunk",
+					  sizeof (GHashNode),
+					  1024, G_ALLOC_ONLY);
+      
+      hash_node = g_chunk_new (GHashNode, node_mem_chunk);
+    }
+  G_UNLOCK (g_hash_global);
+#endif
   
   hash_node->key = key;
   hash_node->value = value;
@@ -718,7 +684,20 @@ g_hash_node_destroy (GHashNode      *hash_node,
     key_destroy_func (hash_node->key);
   if (value_destroy_func)
     value_destroy_func (hash_node->value);
-  g_slice_free (GHashNode, hash_node);
+  
+#ifdef ENABLE_GC_FRIENDLY
+  hash_node->key = NULL;
+  hash_node->value = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
+
+#ifdef DISABLE_MEM_POOLS
+  g_free (hash_node);
+#else
+  G_LOCK (g_hash_global);
+  hash_node->next = node_free_list;
+  node_free_list = hash_node;
+  G_UNLOCK (g_hash_global);
+#endif
 }
 
 static void
@@ -726,18 +705,53 @@ g_hash_nodes_destroy (GHashNode *hash_node,
 		      GFreeFunc  key_destroy_func,
 		      GFreeFunc  value_destroy_func)
 {
+#ifdef DISABLE_MEM_POOLS
   while (hash_node)
     {
       GHashNode *next = hash_node->next;
+
       if (key_destroy_func)
 	key_destroy_func (hash_node->key);
       if (value_destroy_func)
 	value_destroy_func (hash_node->value);
-      g_slice_free (GHashNode, hash_node);
+
+      g_free (hash_node);
       hash_node = next;
+    }  
+#else
+  if (hash_node)
+    {
+      GHashNode *node = hash_node;
+  
+      while (node->next)
+	{
+	  if (key_destroy_func)
+	    key_destroy_func (node->key);
+	  if (value_destroy_func)
+	    value_destroy_func (node->value);
+
+#ifdef ENABLE_GC_FRIENDLY
+	  node->key = NULL;
+	  node->value = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
+
+	  node = node->next;
+	}
+
+      if (key_destroy_func)
+	key_destroy_func (node->key);
+      if (value_destroy_func)
+	value_destroy_func (node->value);
+
+#ifdef ENABLE_GC_FRIENDLY
+      node->key = NULL;
+      node->value = NULL;
+#endif /* ENABLE_GC_FRIENDLY */
+ 
+      G_LOCK (g_hash_global);
+      node->next = node_free_list;
+      node_free_list = hash_node;
+      G_UNLOCK (g_hash_global);
     }
+#endif
 }
-
-
-#define __G_HASH_C__
-#include "galiasdef.c"
