@@ -34,38 +34,12 @@
 #include <stdio.h>
 #include <windows.h>
 
-#ifdef __MSVC__
 #include <tlhelp32.h>
-#else
 
 #ifdef G_WITH_CYGWIN
 #include <sys/cygwin.h>
 #endif
 
-/* The w32api headers supplied with the mingw gcc don't have
- * tlhelp32.h. We really only need the MODULEENTRY32 struct and the
- * TH32CS_SNAPMODULE value, so provide them here.
- */
-
-#define MAX_MODULE_NAME32 255
-
-typedef struct
-{
-  DWORD dwSize;
-  DWORD th32ModuleID;
-  DWORD th32ProcessID;
-  DWORD GlblcntUsage;
-  DWORD ProccntUsage;
-  BYTE  *modBaseAddr; 
-  DWORD modBaseSize; 
-  HMODULE hModule;     
-  char szModule[MAX_MODULE_NAME32 + 1];
-  char szExePath[MAX_PATH];
-} MODULEENTRY32;
-#define TH32CS_SNAPMODULE 8
-
-#endif
-
 static void
 set_error (void)
 {
@@ -78,7 +52,8 @@ set_error (void)
 /* --- functions --- */
 static gpointer
 _g_module_open (const gchar *file_name,
-		gboolean     bind_lazy)
+		gboolean     bind_lazy,
+		gboolean     bind_local)
 {
   HINSTANCE handle;
 #ifdef G_WITH_CYGWIN
@@ -253,16 +228,30 @@ _g_module_build_path (const gchar *directory,
   k = strlen (module_name);
     
   if (directory && *directory)
-    if (k > 4 && g_strcasecmp (module_name + k - 4, ".dll") == 0)
+    if (k > 4 && g_ascii_strcasecmp (module_name + k - 4, ".dll") == 0)
       return g_strconcat (directory, G_DIR_SEPARATOR_S, module_name, NULL);
+#ifdef G_WITH_CYGWIN
+    else if (strncmp (module_name, "lib", 3) == 0 || strncmp (module_name, "cyg", 3) == 0)
+      return g_strconcat (directory, G_DIR_SEPARATOR_S, module_name, ".dll", NULL);
+    else
+      return g_strconcat (directory, G_DIR_SEPARATOR_S, "cyg", module_name, ".dll", NULL);
+#else
     else if (strncmp (module_name, "lib", 3) == 0)
       return g_strconcat (directory, G_DIR_SEPARATOR_S, module_name, ".dll", NULL);
     else
       return g_strconcat (directory, G_DIR_SEPARATOR_S, "lib", module_name, ".dll", NULL);
-  else if (k > 4 && g_strcasecmp (module_name + k - 4, ".dll") == 0)
+#endif
+  else if (k > 4 && g_ascii_strcasecmp (module_name + k - 4, ".dll") == 0)
     return g_strdup (module_name);
+#ifdef G_WITH_CYGWIN
+  else if (strncmp (module_name, "lib", 3) == 0 || strncmp (module_name, "cyg", 3) == 0)
+    return g_strconcat (module_name, ".dll", NULL);
+  else
+    return g_strconcat ("cyg", module_name, ".dll", NULL);
+#else
   else if (strncmp (module_name, "lib", 3) == 0)
     return g_strconcat (module_name, ".dll", NULL);
   else
     return g_strconcat ("lib", module_name, ".dll", NULL);
+#endif
 }
